﻿using System;
using UnityEngine;
using System.Collections;
using PixelCrushers.DialogueSystem;
using Rewired;
using BlueGoo.Utils;

// TODO: NOT USED

public class CustomSkipButton : MonoBehaviour {
	// Default values set before changing them

	[Header("Normal values")]
	[SerializeField] private float minSubtitleSeconds = 2;

	[SerializeField] private float subtitleCharsPerSec = 50;
	[SerializeField] private float typewriterCharsPerSec = 50;
	[SerializeField] private float fullPauseDuration = 1;
	[SerializeField] private float quarterPauseDuration = 0.25f;

	[Header("Fast forward values")]
	[SerializeField] private float skipMinSubtitleSeconds = 1;

	[SerializeField] private float skipSubtitleCharsPerSec = 100;
	[SerializeField] private float skipTypewriterCharsPerSec = 100;
	[SerializeField] private float skipFullPauseDuration = 0.5f;
	[SerializeField] private float skipQuarterPauseDuration = 0f;

	[SerializeField] private float durationToWaitWhenComplete = 2f;

	[Tooltip("Dialogue UI that the continue button affects.")]
	//public StandardDialogueUI dialogueUI;
	public StandardUISubtitlePanel uiSubtitlePanel;

	//private StandardDialogueUI dialogueUI;
	private CustomUnityUITypewriterEffect typewriter;

	private bool skipping;
	private Coroutine skipDialogueCoroutine;

	[SerializeField]
	private bool debugConversationEvents;
	
	private void SetTypewriter() {
		//dialogueUI = GetComponentInChildren<StandardDialogueUI>();
		//typewriter = dialogueUI.conversationUIElements.defaultNPCSubtitlePanel.subtitleText.gameObject.GetComponent<UnityUITypewriterEffect>();
		typewriter = uiSubtitlePanel.GetComponentInChildren<CustomUnityUITypewriterEffect>();
		if (typewriter == null) Debug.LogError("UnityUITypewriterEffect component missing");
	}

	private void Start() {
		// Subscribe to Rewired actions
		foreach (var player in MasterManager.GameManager.players) {
			player.AddInputEventDelegate(OnEscapeButtonDown, UpdateLoopType.Update, InputActionEventType.ButtonJustPressed, "Escape");
		}
		subtitleCharsPerSec = MasterManager.ConfigManager.charactersPerSecond;
		fullPauseDuration = MasterManager.ConfigManager.fullPauseDuration;
		quarterPauseDuration = MasterManager.ConfigManager.quarterPauseDuration;
		
	}

	private void OnDestroy() {
		// Unsubscribe from events when object is destroyed
		foreach (var player in MasterManager.GameManager.players) {
			player.RemoveInputEventDelegate(OnEscapeButtonDown);
		}
	}

	private void OnEscapeButtonDown(InputActionEventData data) {
		Debug.Log("Escape DIALOG Pressed!" + data);
		SkipDialogue();
	}

	private void Init() {
		// Set current dialogueUI to skip (Find conversation with player)

		if (DialogueManager.currentConversationState == null) {
			Debug.Log("No currentConversationState");
			return;
		}
		var currentSpeaker = DialogueManager.currentConversationState.subtitle.speakerInfo.transform;
		if (currentSpeaker == null) return;


		uiSubtitlePanel = currentSpeaker.GetComponentInChildren<CustomStandardUISubtitlePanel>();
		if (uiSubtitlePanel == null) {
			Debug.Log("No uiSubtitlePanel found");
			return;
		}

		// Get typewriter component
		SetTypewriter();

		// This would only be neccesary if multiple conversation can happen at the same time (probably not?)
		/*
		var playerActorTransform = MasterManager.GameManager.characterRpg.transform.parent
			.GetComponentInChildren<DialogueActor>().transform;

		ConversationView matchedConversationView;
		foreach (var conversationRecord in DialogueManager.instance.activeConversations) {
			if (conversationRecord.actor == playerActorTransform || conversationRecord.conversant == playerActorTransform) {
				//dialogueUI = conversation.ConversationView.transform;
				Debug.Log(conversationRecord.ConversationView.transform.name);
				//matchedConversationView = conversationRecord.conversationView;
				//matchedConversationView.
			}
		}
		*/
	}

	// Run when clicking "button" (hidden overlay cover)
	public void SkipDialogue() {
		Debug.Log("SKIP");
		Init();
		Continue();
		return;
		
		// If pressed twice, don't wait for typewriter to finish or its delay
		if (skipping) {
			Continue();
		}

		skipDialogueCoroutine = StartCoroutine(nameof(SkipDialogueSequence));
	}

	private void Continue() {
		Debug.Log("<color=magenta>" + uiSubtitlePanel.name + " CONTINUE</color>");
		skipping = false;
		//dialogueUI.OnContinue();
		uiSubtitlePanel.OnContinue();
		//AbortSkipDialogueCoroutine();
	}

	private void AbortSkipDialogueCoroutine() {
		if (skipDialogueCoroutine != null) StopCoroutine(skipDialogueCoroutine);
	}
	
	IEnumerator SkipDialogueSequence() {
		skipping = true;

		FastForwardTypewriter();

		var timeout = Time.time + durationToWaitWhenComplete;

		while (skipping && Time.time < timeout) {
			yield return null;
		}

		/*
		if (skipping) {
			// Reset typewriter and close the dialogue (Only if it hasn't been closed already)
			ResetTypewriter();
			Continue();
		}
		*/
	}

	private void FastForwardTypewriter() {
		if (typewriter == null) Init();
		Debug.Log(Time.time.ToString("F1") + $": <color=yellow>FastForward Typewriter ({typewriter.charactersPerSecond} -> {skipTypewriterCharsPerSec})</color>");

		// Set values to fast forward:
		typewriter.charactersPerSecond = skipTypewriterCharsPerSec;
		typewriter.fullPauseDuration = skipFullPauseDuration;
		typewriter.quarterPauseDuration = skipQuarterPauseDuration;
		DialogueManager.DisplaySettings.subtitleSettings.subtitleCharsPerSecond = skipSubtitleCharsPerSec;
		DialogueManager.DisplaySettings.subtitleSettings.minSubtitleSeconds = skipMinSubtitleSeconds;
		//DialogueManager.DisplaySettings.subtitleSettings.continueButton = DisplaySettings.SubtitleSettings.ContinueButtonMode.Never;
	}

	private void ResetTypewriter() {
		if (typewriter == null) Init();
		Debug.Log(Time.time.ToString("F1") + $": <color=green>RESET Typewriter ({typewriter.charactersPerSecond} -> {typewriterCharsPerSec})</color>");
		//Debug.Log(Time.time.ToString("F1") + ": <color=green>Reset Typewriter</color>");

		// Set original values:
		typewriter.charactersPerSecond = typewriterCharsPerSec;
		typewriter.fullPauseDuration = fullPauseDuration;
		typewriter.quarterPauseDuration = quarterPauseDuration;
		DialogueManager.DisplaySettings.subtitleSettings.subtitleCharsPerSecond = subtitleCharsPerSec;
		DialogueManager.DisplaySettings.subtitleSettings.minSubtitleSeconds = minSubtitleSeconds;
		//DialogueManager.DisplaySettings.subtitleSettings.continueButton = DisplaySettings.SubtitleSettings.ContinueButtonMode.Always;
	}

	private void OnConversationResponseMenu(Response[] responses) {
		if (debugConversationEvents) Debug.Log("<color=cyan>OnConversationResponseMenu</color>");
	}

	// Reset typewriter for each new line
	private void OnConversationLine(Subtitle subtitle) {
		if (subtitle.dialogueEntry.DialogueText.Length == 0) return;
		ResetTypewriter();
		if (debugConversationEvents) Debug.Log("<color=cyan>OnConversationLine</color> " + subtitle.speakerInfo.Name + 
		          ": " + subtitle.dialogueEntry.DialogueText.Substring(0,10) + ".. (" + subtitle.dialogueEntry.DialogueText.Length + ")");
	}

	private void OnConversationLineEnd(Subtitle subtitle) {
		if (debugConversationEvents) Debug.Log("<color=cyan>OnConversationLineEnd </color>" + subtitle.speakerInfo.Name);
		ResetTypewriter();
	}

	private void OnConversationStart(Transform actor) {
		if (debugConversationEvents) Debug.Log("<color=cyan>OnConversationStart </color>" + actor.name);
	}

	private void OnConversationEnd(Transform actor) {
		if (debugConversationEvents) Debug.Log("<color=cyan>OnConversationEnd </color>" + actor.name);
	}

	private void OnConversationCancelled(Transform actor) {
		if (debugConversationEvents) Debug.Log("<color=cyan>OnConversationCancelled </color>" + actor.name);
	}

	private void OnBarkStart(Transform actor) {
		if (debugConversationEvents) Debug.Log("<color=cyan>OnBarkStart </color>" + actor.name);
	}

	private void OnBarkEnd(Transform actor) {
		if (debugConversationEvents) Debug.Log("<color=cyan>OnBarkEnd </color>" + actor.name);
	}
}